#!/bin/bash

# dovecot-deduper
#
# This script removes mail duplicates of your IMAP server on a by-folder basis.
#
# * Requires IMAP folders in a filesystem it can access
# * Requires awk, bash, find, grep, md5sum, perl, sed
#
# (c) 2013 by Frederic Krueger / dovecot-deduper@dev.techno.holics.at
#
# Licensed under Creative Commons Attribution-ShareAlike 3.0 Unported
# There is no warranty of any kind, explicit or implied, for anything this software does or does not do.
# 
# Updates for this piece of software could be available under the following URL:
#   http://dev.techno.holics.at/dovecot-deduper/

BASEDIR=""
DOIT="no"

### DONT edit below here \/

### TODO keep the first mail of a list of dupes, not the last one. this is to avoid getting unread mails you've already read before adding the dupe to the maildir.

OURFNAME="${0##*/}"

echo ""
if [ "x$1" == "x" ] || [ ! -d "$1" ]; then
  echo "usage: $OURFNAME <users-maildir> [doit]"
  echo ""
  echo "   Scans all folders in <users-maildir> and deletes all (but one) dupe messages found per folder"
  echo "   (using Message-Id or md5sum)"
  echo ""
  echo "   Set second parameter to 'doit' to actually delete files."
  echo ""
  echo "(c) 2013 by Frederic Krueger / dovecot-deduper@dev.techno.holics.at"
  echo ""
  echo "Licensed under Creative Commons Attribution-ShareAlike 3.0 Unported"
  echo "There is no warranty of any kind, explicit or implied, for anything this software does or does not do."
  echo ""
  exit
fi

MYNAME="dovecotdeduper"

[ "x$1" != "x" ] && BASEDIR="$1"
[ "x$2" != "x" ] && DOIT="$2"

echo ". Looking for cur/ folders in users-maildir $BASEDIR"

TMP=`mktemp /tmp/${MYNAME}-maildirs.XXXXX.tmp`
find $BASEDIR -type d -name "cur" | sort -u > $TMP

TMP2=`mktemp /tmp/${MYNAME}-maildir-contents.XXXXX.tmp`
TMP3=`mktemp /tmp/${MYNAME}-maildir-contents-sorted.XXXXX.tmp`
while read mdir; do
  echo ".. Processing maildir $mdir"

  NUMDIRFILES=`ls -1 $mdir | wc -l`
  if [ "$NUMDIRFILES" -gt 0 ]; then
    echo "... Getting Message-Ids from $NUMDIRFILES mails in directory"
    echo >$TMP2
    for mfile in $mdir/*; do
      MSGID=`grep -i '^ *Message-Id:' $mfile | sed -r 's/^.*<([^>]*)>.*$/\1/g'`
      if [ "x$MSGID" == "x" ]; then       # for messages with Message-Id, create md5sum of message content without X- Flags.
        MSGID=`grep -vP '(^X-DSPAM|\!DSPAM|.*for .* .single-drop.)' $mfile | md5sum | cut -d' ' -f1`
         echo ".... Got mfile $mfile (had to use md5sum)"
#      else
#        echo ".... Got mfile $mfile (msgid $MSGID)"
      fi
      echo "$MSGID $mfile" >>$TMP2
    done
  
    echo "... Sorting ids gotten and return all but the first entry"
    cat $TMP2 | sort | perl -nle 'BEGIN { our %foo = (); } my @t=split/ +/, $_; if (($#t == 1) and (defined($t[0])) and (defined($t[1]))) { if (!defined($foo{$t[0]})) { $foo{$t[0]} = (); } chomp($t[1]); push @{$foo{$t[0]}}, $t[1]; } END { foreach my $k (sort keys %foo) { next if ((!defined($k)) or ($k eq "")); if ($#{$foo{$k}} > 0) { for (my $i=1; $i <= $#{$foo{$k}}; $i++) { if (0) {print "key $k, i " .($i+1). "/" .($#{$foo{$k}}+1). ": " .${$foo{$k}}[$i]. ""; } print "" .${$foo{$k}}[$i]; } } } }' >$TMP3
    echo "... Deleting duplicate mailfiles in directory"
    while read mfile; do
      if [ "x$DOIT" == "xdoit" ]; then
        echo ".... Deleting dupe mail: $mfile"
        rm -f $mfile
      else
        echo ".... NOT deleting dupe mail: $mfile"
      fi
    done < $TMP3
  else
    echo "... Nothing to do in directory"
  fi
done < $TMP

# clearing up tempfiles
echo ". Cleaning up our current tempfiles ($TMP $TMP2 $TMP3)"
rm -f $TMP $TMP2 $TMP3
echo ". Cleaning up our tempfiles older than two days"
find /tmp -mtime +2 -iname "${MYNAME}-*.tmp" -print -exec rm -f {} \;

echo ". fin!"
echo ""
#fin? You from funland?
